/***************************************************************************
 *   copyright       : (C) 2006 by Pascal Brachet                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "bookmarkdialog.h"

#include <QDebug>

BookmarkDialog::BookmarkDialog(QWidget *parent)
    :QDialog( parent)
{
ui.setupUi(this);
movieToPlay="";
currentMovieLocation="";

connect(ui.listWidget, SIGNAL(itemSelectionChanged()),this, SLOT(updateItemInfo()));
connect(ui.lineEditName, SIGNAL(textChanged(const QString &)),this,SLOT(updateName()));
connect(ui.lineEditLocation, SIGNAL(textChanged(const QString &)),this,SLOT(updateLocation()));
connect(ui.playButton, SIGNAL(clicked()), this, SLOT(playSelectedMovie()));
connect(ui.newButton, SIGNAL(clicked()), this, SLOT(addItem()));
connect(ui.deleteButton, SIGNAL(clicked()), this, SLOT(deleteItem()));
connect(ui.upButton, SIGNAL(clicked()), this, SLOT(upItem()));
connect(ui.downButton, SIGNAL(clicked()), this, SLOT(downItem()));
connect(ui.currentButton, SIGNAL(clicked()), this, SLOT(addCurrentMovie()));

ui.listWidget->clear();
ui.listWidget->setAlternatingRowColors(true);
ui.listWidget->setSelectionMode (QAbstractItemView::SingleSelection);
}

BookmarkDialog::~BookmarkDialog(){
}

//************************************************************
void BookmarkDialog::Init()
{
for (int i = 0; i < nameList.count(); i++)
	{
	if (!nameList.at(i).isEmpty() && !locationList.at(i).isEmpty())
		{
		QListWidgetItem *newItem = new QListWidgetItem(ui.listWidget);
		newItem->setText(nameList.at(i));
		if (locationList.at(i).lastIndexOf(QRegExp("\\.mp3|\\.wav|\\.ogg|\\.mka|\\.wma|\\.mp2|\\.ra|\\.asf"))>-1) newItem->setIcon(QIcon(":/images/smallmusic.png"));
		else if (locationList.at(i).lastIndexOf(QRegExp("\\.m3u|\\.pls|\\.asx|\\.wax|\\.wvx|\\.ram"))>-1) newItem->setIcon(QIcon(":/images/smallplaylist.png"));
		else newItem->setIcon(QIcon(":/images/smallvideo.png"));
		newItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
		}
	else
		{
		nameList.removeAt(i);
		locationList.removeAt(i);
		}
	}
if (ui.listWidget->count()>0)
	{
	ui.listWidget->setCurrentItem(ui.listWidget->item(0));
	ui.listWidget->setItemSelected(ui.listWidget->currentItem(), true);
	}
}
//************************************************************
void BookmarkDialog::playSelectedMovie()
{
int current=getItem();
if (current<0) movieToPlay="";
else movieToPlay=locationList[current];
if (!movieToPlay.isEmpty()) accept();
}

void BookmarkDialog::addItem()
{
nameList.append("New Bookmark (edit me)");
locationList.append("");
QListWidgetItem *newItem = new QListWidgetItem(ui.listWidget);
newItem->setText("New Bookmark (edit me)");
newItem->setIcon(QIcon(":/images/smallvideo.png"));
newItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
ui.listWidget->setCurrentItem(newItem);
ui.listWidget->setItemSelected(ui.listWidget->currentItem(), true);
ui.lineEditName->setFocus();
}

void BookmarkDialog::deleteItem()
{
int current=getItem();
if (current<0) return;
delete ui.listWidget->currentItem();
nameList.removeAt(current);
locationList.removeAt(current);
ui.listWidget->setCurrentItem(ui.listWidget->item(current));
ui.listWidget->setItemSelected(ui.listWidget->item(current),true);
updateItemInfo();
}

void BookmarkDialog::upItem()
{
int current=getItem();
if (current<=0) return;
QListWidgetItem *item=ui.listWidget->item(current)->clone();
delete ui.listWidget->item(current);
ui.listWidget->insertItem(current-1,item);
nameList.swap(current,current-1);
locationList.swap(current,current-1);
ui.listWidget->setCurrentItem(ui.listWidget->item(current-1));
ui.listWidget->setItemSelected(ui.listWidget->item(current-1),true);
}

void BookmarkDialog::downItem()
{
int current=getItem();
if (current>=ui.listWidget->count()-1 || current<0) return;
QListWidgetItem *item=ui.listWidget->item(current)->clone();
delete ui.listWidget->item(current);
ui.listWidget->insertItem(current+1,item);
nameList.swap(current,current+1);
locationList.swap(current,current+1);
ui.listWidget->setCurrentItem(ui.listWidget->item(current+1));
ui.listWidget->setItemSelected(ui.listWidget->item(current+1),true);
}

void BookmarkDialog::addCurrentMovie()
{
if (currentMovieLocation.isEmpty()) return;
nameList.append("New Bookmark (edit me)");
locationList.append(currentMovieLocation);
QListWidgetItem *newItem = new QListWidgetItem(ui.listWidget);
newItem->setText("New Bookmark (edit me)");
if (currentMovieLocation.lastIndexOf(QRegExp("\\.mp3|\\.wav|\\.ogg|\\.mka|\\.wma|\\.mp2|\\.ra|\\.asf"))>-1) newItem->setIcon(QIcon(":/images/smallmusic.png"));
else if (currentMovieLocation.lastIndexOf(QRegExp("\\.m3u|\\.pls|\\.asx|\\.wax|\\.wvx|\\.ram"))>-1) newItem->setIcon(QIcon(":/images/smallplaylist.png"));
else newItem->setIcon(QIcon(":/images/smallvideo.png"));
newItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
ui.listWidget->setCurrentItem(newItem);
ui.listWidget->setItemSelected(ui.listWidget->currentItem(), true);
ui.lineEditName->setFocus();
}
//*****************************************************************
void BookmarkDialog::updateItemInfo()
{
int current=getItem();
if (current<0) 
	{
	ui.lineEditName->setText("");
	ui.lineEditLocation->setText("");
	}
else
	{
	ui.lineEditName->setText(nameList[current]);
	ui.lineEditLocation->setText(locationList[current]);
	}
}

void BookmarkDialog::updateName()
{
int current=getItem();
if (current<0) return;
ui.listWidget->item(current)->setText(ui.lineEditName->text());
nameList[current]=ui.lineEditName->text();
}

void BookmarkDialog::updateLocation()
{
int current=getItem();
if (current<0) return;
locationList[current]=ui.lineEditLocation->text();
}

int BookmarkDialog::getItem()
{
QList<QListWidgetItem *> items;
items = ui.listWidget->selectedItems();
if (items.count() > 0) 
	{
	ui.listWidget->setCurrentItem(items[0]);
	return ui.listWidget->row(items[0]);
	}
return -1;
}
