/***************************************************************************
 *   copyright       : (C) 2006 by Pascal Brachet                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <QFileDialog>
#include <QFileInfo>
#include <QTextStream>
#include <QFile>
#include <QDir>
#include <QMessageBox>
#include <QDebug>
#include <QHeaderView>
#include <QScrollBar>

#include "playlistdialog.h"

PlaylistDialog::PlaylistDialog(QWidget *parent)
    :QDialog( parent)
{
ui.setupUi(this);

mediaFilter= QString("*.mpg *.mpeg *.wmv *.avi *.asf *.mov *.rm *.divx *.mkv *.ogm *.mp3 *.wav *.ogg *.mka *.wma *.mp2 *.ra").split(" ");

connect(ui.fileButton, SIGNAL( clicked() ), this, SLOT( actionAddFile() ) );
connect(ui.folderButton, SIGNAL( clicked() ), this, SLOT( actionAddFolder() ) );
connect(ui.urlButton, SIGNAL( clicked() ), this, SLOT( actionAddURL() ) );
connect(ui.upButton, SIGNAL( clicked() ), this, SLOT( actionUp() ) );
connect(ui.downButton, SIGNAL( clicked() ), this, SLOT( actionDown() ) );
connect(ui.deleteButton, SIGNAL( clicked() ), this, SLOT( actionDelete() ) );
connect(ui.openButton, SIGNAL( clicked() ), this, SLOT( fileOpen() ) );
connect(ui.saveButton, SIGNAL( clicked() ), this, SLOT( fileSave() ) );
connect(ui.newButton, SIGNAL( clicked() ), this, SLOT( fileNew() ) );
connect(ui.playButton, SIGNAL( clicked() ), this, SLOT( playPlaylist() ) );

modified=false;
playlistname="untitled";
setWindowTitle(tr("Playlist Editor : ")+playlistname);
ui.listWidget->clear();
ui.listWidget->setAlternatingRowColors(true);
mediacontent.clear();
tempPlayFile="";

ui.dirWidget->setSelectionMode (QAbstractItemView::ExtendedSelection);
ui.dirWidget->setViewMode(QListView::ListMode);
connect(ui.dirWidget,SIGNAL(itemDoubleClicked(QListWidgetItem*)),this,SLOT(doubleclick(QListWidgetItem*)));
QDir::setCurrent(QDir::homePath());
updateDirWidget();
}

PlaylistDialog::~PlaylistDialog(){
}

void PlaylistDialog::closeEvent(QCloseEvent *e)
{
if (modified) 
	{
	switch(  QMessageBox::warning(this,tr("Playlist Editor : "),
					tr("The current playlist has been modified. "
					"Do you want to save it before closing?"),
					tr("Save"), tr("Don't Save"), tr("Cancel"),
					0,
					2 ) )
		{
		case 0:
		fileSave();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
e->accept();
}

void PlaylistDialog::accept()
{
if (modified) 
	{
	switch(  QMessageBox::warning(this,tr("Playlist Editor : "),
					tr("The current playlist has been modified. "
					"Do you want to save it before closing?"),
					tr("Save"), tr("Don't Save"), tr("Cancel"),
					0,
					2 ) )
		{
		case 0:
		fileSave();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QDialog::accept();
}

void PlaylistDialog::fileOpen()
{
if (modified) 
	{
	switch(  QMessageBox::warning(this,tr("Playlist Editor : "),
					tr("The current playlist has been modified. "
					"Do you want to save it before closing?"),
					tr("Save"), tr("Don't Save"), tr("Cancel"),
					0,
					2 ) )
		{
		case 0:
		fileSave();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
QString line;
int mediacounter=-1;
ui.listWidget->clear();
mediacontent.clear();
QString currentDir=QDir::homePath();
playlistname = QFileDialog::getOpenFileName(this,tr("Open File"),currentDir,"Playlist files (*.m3u)");
QFile playlistfile(playlistname);
if (!playlistfile.open(QIODevice::ReadOnly | QIODevice::Text))
	{
	QMessageBox::warning( this,tr("Error"),tr("Qxmp can't read this playlist."));
	return;
	}
QTextStream stream(&playlistfile);
line = stream.readLine();
if (line.startsWith("#EXTM3U"))
	{
	while (!stream.atEnd()) 
		{
		line = stream.readLine();
		if (line.startsWith("#EXTINF"))
			{
			ui.listWidget->addItem(line.right(line.length()-line.indexOf(",")-1));	
			mediacounter++;
			mediacontent.append(QStringList(""));
			mediacontent[mediacounter]<<line+"\n";
			}
		else
			{
			if (mediacounter>-1) mediacontent[mediacounter]<<line+"\n";
			}
		}
	setWindowTitle(tr("Playlist Editor : ")+playlistname);
	}
else QMessageBox::warning( this,tr("Error"),tr("Qxmp can't read this playlist."));
playlistfile.close();
}

void PlaylistDialog::fileSave()
{
if (playlistname=="untitled" ) 
	{
	QString currentDir=QDir::homePath();
	playlistname = QFileDialog::getSaveFileName(this,tr("Save As"),currentDir,"Playlist files (*.m3u)");
	}
QFile playlistfile(playlistname);
if(!playlistfile.open(QIODevice::WriteOnly))
	{
	QMessageBox::warning( this,tr("Error"),tr("The file could not be saved. Please check if you have write permission."));
	return;
	}
playlistfile.write(QString("#EXTM3U\n").toAscii(),QString("#EXTM3U\n").length());
for (int n=0;n<ui.listWidget->count();n++)
	{
	playlistfile.write(mediacontent[n].join("").toAscii(),mediacontent[n].join("").toAscii().length());
	}
playlistfile.close();
modified=false;
setWindowTitle(tr("Playlist Editor : ")+playlistname);
}

void PlaylistDialog::fileNew()
{
if (modified) 
	{
	switch(  QMessageBox::warning(this, tr("Playlist Editor : "),
					tr("The current playlist has been modified. "
					"Do you want to save it before closing?"),
					tr("Save"), tr("Don't Save"), tr("Cancel"),
					0,
					2 ) )
		{
		case 0:
		fileSave();
		break;
		case 1:
		break;
		case 2:
		default:
		return;
		break;
		}
	}
ui.listWidget->clear();
mediacontent.clear();
modified=false;
playlistname="untitled";
setWindowTitle(tr("Playlist Editor : ")+playlistname);
}

void PlaylistDialog::addItem(QString name,QString fn)
{
modified=true;
setWindowTitle(tr("Playlist Editor : ")+playlistname+" [modified]");
ui.listWidget->addItem(name);
int mediacounter=ui.listWidget->count()-1;
mediacontent.append(QStringList(""));
mediacontent[mediacounter]<<"#EXTINF:0,"<<name<<"\n"<<fn<<"\n";
if (ui.listWidget->count()>0)
	{
	ui.listWidget->setCurrentItem(ui.listWidget->item(ui.listWidget->count()-1));
	ui.listWidget->setItemSelected(ui.listWidget->currentItem(), true);
	}
}

void PlaylistDialog::actionUp()
{
int current=ui.listWidget->currentRow();
if (current<=0) return;
modified=true;
setWindowTitle(tr("Playlist Editor : ")+playlistname+" [modified]");
QListWidgetItem *item=ui.listWidget->item(current)->clone();
delete ui.listWidget->item(current);
ui.listWidget->insertItem(current-1,item);
mediacontent.swap(current,current-1);
ui.listWidget->setCurrentItem(ui.listWidget->item(current-1));
ui.listWidget->setItemSelected(ui.listWidget->item(current-1),true);
}

void PlaylistDialog::actionDown()
{
int current=ui.listWidget->currentRow();
if (current>=ui.listWidget->count()-1 || current<0) return;
modified=true;
setWindowTitle(tr("Playlist Editor : ")+playlistname+" [modified]");
QListWidgetItem *item=ui.listWidget->item(current)->clone();
delete ui.listWidget->item(current);
ui.listWidget->insertItem(current+1,item);
mediacontent.swap(current,current+1);
ui.listWidget->setCurrentItem(ui.listWidget->item(current+1));
ui.listWidget->setItemSelected(ui.listWidget->item(current+1),true);
}

void PlaylistDialog::actionDelete()
{
int current=ui.listWidget->currentRow();
if (current<0) return;
delete ui.listWidget->currentItem();
mediacontent.removeAt(current);
modified=true;
setWindowTitle(tr("Playlist Editor : ")+playlistname+" [modified]");
if (current==ui.listWidget->count())
	{
	ui.listWidget->setCurrentItem(ui.listWidget->item(current-1));
	ui.listWidget->setItemSelected(ui.listWidget->item(current-1),true);
	}
else
	{
	ui.listWidget->setCurrentItem(ui.listWidget->item(current));
	ui.listWidget->setItemSelected(ui.listWidget->item(current),true);
	}
}

void PlaylistDialog::actionAddURL()
{
QString fn,name;
urlDlg = new PlaylistUrlDialog(this);
if (urlDlg->exec())
	{
	fn=urlDlg->ui.lineEditURL->text();
	name=urlDlg->ui.lineEditName->text();
	}
delete urlDlg;
if (!fn.isEmpty()) addItem(name,fn);
}

void PlaylistDialog::actionAddFile()
{
QString fn;
QList<QListWidgetItem *> items;
items = ui.dirWidget->selectedItems();
for (int i = 0; i < items.count(); ++i) 
	{
	fn=cwd+QString("/")+items[i]->text();
	if (!fn.endsWith(QString("/"))) addItem(QFileInfo(fn).fileName(),fn);
	}
}

void PlaylistDialog::actionAddFolder()
{
int current=ui.dirWidget->currentRow();
if (current<0) return;
QString fn=cwd+QString("/")+ui.dirWidget->currentItem()->text();
if (!fn.endsWith(QString("/")) || fn=="../") return;
QFileInfoList folderlist =QDir(fn).entryInfoList(mediaFilter,QDir::Files,QDir::Name);
for (int i = 0; i < folderlist.count(); ++i) 
	{
	QFileInfo fi=folderlist.at(i);
	addItem(fi.fileName(),fi.filePath());
	}
}

//**************************************************
void PlaylistDialog::updateDirWidget()
{
ui.dirWidget->clear();
cwd=QDir::currentPath();
QFileInfoList foo=QDir(cwd).entryInfoList(mediaFilter,QDir::AllEntries|QDir::AllDirs,QDir::DirsFirst);
QFileInfoList::const_iterator it(foo.constBegin());
while(it!=foo.constEnd())
	{
	if ((*it).fileName()!=".")
		{
		if((*it).isDir()) 
			{
			QListWidgetItem *newItem = new QListWidgetItem(ui.dirWidget);
			newItem->setText(((*it).fileName()+QString("/")));
			newItem->setIcon(QIcon(":/images/smalldir.png"));
			newItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
			}
		else 
			{
			QListWidgetItem *newItem = new QListWidgetItem(ui.dirWidget);
			newItem->setText(((*it).fileName()));
			if (((*it).fileName()).lastIndexOf(QRegExp("\\.mp3|\\.wav|\\.ogg|\\.mka|\\.wma|\\.mp2|\\.ra|\\.asf"))>-1) newItem->setIcon(QIcon(":/images/smallmusic.png"));
			else newItem->setIcon(QIcon(":/images/smallvideo.png"));
			newItem->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled);
			}
		}
	it++;
	}
if (ui.dirWidget->count()>0)
	{
	ui.dirWidget->setCurrentItem(ui.dirWidget->item(0));
	ui.dirWidget->setItemSelected(ui.dirWidget->currentItem(), true);
	}
}

void PlaylistDialog::doubleclick(QListWidgetItem* item)
{
QString filename=item->text();
if(filename.endsWith(QString("/")))
	{
	QDir::setCurrent(cwd+QString("/")+filename);
	updateDirWidget();
	}
else addItem(QFileInfo(filename).fileName(),filename);
}

void PlaylistDialog::playPlaylist()
{
tempPlayFile="";
if (ui.listWidget->count()>0)
	{
	for (int n=0;n<ui.listWidget->count();n++)
		{
		tempPlayFile+=mediacontent[n].join("");
		}
	}
accept();
}

