/***************************************************************************
 *   copyright       : (C) 2006 by Pascal Brachet                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "qxmp.h"

#include <QFileDialog>
#include <QDesktopWidget>
#include <QPalette>
#include <QApplication>
#include <QRegExp>
#include <QDebug>
#include <QSettings>
#include <QCloseEvent>
#include <QMessageBox>
#include <QFileInfo>
#include <QStyleFactory>
#include <QStyle>
#include <QTextStream>
#include <QTime>
#include <QFontDatabase>
#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>
#include <QDomNodeList>

#include <sys/ioctl.h>
#include <fcntl.h>
#include <sys/soundcard.h>
#include <math.h>
#include <sys/types.h>

qxmp::qxmp(QWidget *parent)
    :QDialog( parent)
{
QApplication::setStyle(QStyleFactory::create("plastique"));
readSettings();
ui.setupUi(this);

ui.playButton->setImages("play");
ui.playButton->setToolTip("Open and Play Media File");
connect(ui.playButton, SIGNAL( clicked() ), this, SLOT( loadFile() ) );
ui.pauseButton->setImages("pause");
ui.pauseButton->setToolTip("Pause [Space Bar]");
connect(ui.pauseButton, SIGNAL( clicked() ), this, SLOT( playerPause() ) );
ui.stopButton->setImages("stop");
ui.stopButton->setToolTip("Stop [S]");
connect(ui.stopButton, SIGNAL( clicked() ), this, SLOT( playerStop() ) );
ui.soundButton->setImages("sound");
ui.soundButton->setToolTip("Mute [M]");
connect(ui.soundButton, SIGNAL( clicked() ), this, SLOT( playerMute() ) );
ui.quitButton->setToolTip("Quit");
ui.quitButton->setImages("quit");
connect(ui.quitButton, SIGNAL( clicked() ), this, SLOT( close() ) );
ui.reloadButton->setImages("replay");
ui.reloadButton->setToolTip("Replay Previous Media");
connect(ui.reloadButton, SIGNAL( clicked() ), this, SLOT( playerReplay() ) );
ui.fwdButton->setImages("forward");
ui.fwdButton->setToolTip("Forward +10% [Right]");
connect(ui.fwdButton, SIGNAL( clicked() ), this, SLOT( playerForward() ) );
ui.rwdButton->setImages("reward");
ui.rwdButton->setToolTip("Reward -10% [Left]");
connect(ui.rwdButton, SIGNAL( clicked() ), this, SLOT( playerReward() ) );
ui.dvdButton->setImages("dvd");
ui.dvdButton->setToolTip("Play DVD");
connect(ui.dvdButton, SIGNAL( clicked() ), this, SLOT( loadDVD() ) );
ui.cddaButton->setImages("cdda");
ui.cddaButton->setToolTip("Play Audio CD");
connect(ui.cddaButton, SIGNAL( clicked() ), this, SLOT( loadCDDA() ) );
ui.configButton->setImages("configure");
ui.configButton->setToolTip("Configure");
connect(ui.configButton, SIGNAL( clicked() ), this, SLOT( configurePlayer() ) );
ui.aboutButton->setImages("help");
ui.aboutButton->setToolTip("Help/About");
connect(ui.aboutButton, SIGNAL( clicked() ), this, SLOT( aboutPlayer() ) );
ui.urlButton->setImages("web");
ui.urlButton->setToolTip("Play URL");
connect(ui.urlButton, SIGNAL( clicked() ), this, SLOT( loadURL() ) );
ui.bookmarkButton->setImages("bookmark");
ui.bookmarkButton->setToolTip("Bookmark Manager");
connect(ui.bookmarkButton, SIGNAL( clicked() ), this, SLOT( manageBookmarks() ) );
ui.playlistButton->setImages("playlist");
ui.playlistButton->setToolTip("Playlist Editor");
connect(ui.playlistButton, SIGNAL( clicked() ), this, SLOT( playlistEditor() ) );
ui.nextButton->setImages("next");
ui.nextButton->setToolTip("Next [CTRL+Right]");
connect(ui.nextButton, SIGNAL( clicked() ), this, SLOT( playerNext() ) );
ui.previousButton->setImages("previous");
ui.previousButton->setToolTip("Previous [CTRL+Left]");
connect(ui.previousButton, SIGNAL( clicked() ), this, SLOT( playerPrevious() ) );
ui.showlistButton->setImages("showlist");
ui.showlistButton->setToolTip("Show Playlist");
connect(ui.showlistButton, SIGNAL( clicked() ), this, SLOT( showPlaylist() ) );
ui.showplayerButton->setImages("showplayer");
ui.showplayerButton->setToolTip("Show Player");
connect(ui.showplayerButton, SIGNAL( clicked() ), this, SLOT( showPlayer() ) );

pauseAction=new QAction(this);
pauseAction->setShortcut(Qt::Key_Space);
connect(pauseAction, SIGNAL(triggered()),this, SLOT(playerPause()));
this->addAction(pauseAction);

stopAction=new QAction(this);
stopAction->setShortcut(Qt::Key_S);
connect(stopAction, SIGNAL(triggered()),this, SLOT(playerStop()));
this->addAction(stopAction);

muteAction=new QAction(this);
muteAction->setShortcut(Qt::Key_M);
connect(muteAction, SIGNAL(triggered()),this, SLOT(playerMute()));
this->addAction(muteAction);

fwdAction=new QAction(this);
fwdAction->setShortcut(Qt::Key_Right);
connect(fwdAction, SIGNAL(triggered()),this, SLOT(playerForward()));
this->addAction(fwdAction);

rwdAction=new QAction(this);
rwdAction->setShortcut(Qt::Key_Left);
connect(rwdAction, SIGNAL(triggered()),this, SLOT(playerReward()));
this->addAction(rwdAction);

nextAction=new QAction(this);
nextAction->setShortcut(Qt::CTRL+Qt::Key_Right);
connect(nextAction, SIGNAL(triggered()),this, SLOT(playerNext()));
this->addAction(nextAction);

previousAction=new QAction(this);
previousAction->setShortcut(Qt::CTRL+Qt::Key_Left);
connect(previousAction, SIGNAL(triggered()),this, SLOT(playerPrevious()));
this->addAction(previousAction);

connect(this, SIGNAL(playdone()), this, SLOT(playNextMedia()));
connect(this, SIGNAL(infodone()), this, SLOT(startPlayer()));

QPalette palette;
palette.setColor(ui.frame->backgroundRole(), QColor(0,0,0));
ui.frame->setPalette(palette);
ui.frame->setAutoFillBackground(true);

titlewidget=new TitleArea(ui.controlframe,displayFont);

palette.setColor(ui.labelStatus->backgroundRole(), QColor(24,93,140));
palette.setColor(ui.labelStatus->foregroundRole(), QColor(255,255,255));
ui.labelStatus->setPalette(palette);
ui.labelStatus->setAutoFillBackground(true);
ui.labelTime->setPalette(palette);
ui.labelTime->setAutoFillBackground(true);

QFont font(displayFont,12);
font.setBold(true);
font.setItalic(false);
font.setUnderline(false);
font.setStrikeOut(false);
font.setPixelSize(20);

ui.labelStatus->setFont(font);
ui.labelTime->setFont(font);

_mixerfd = open(mixerDevice.toAscii(), O_RDWR);
ui.soundSlider->setToolTip("Volume");
ui.soundSlider->setValue(getVolume());
connect(ui.soundSlider, SIGNAL(valueChanged(int)),this, SLOT(setVolume(int)));

ui.stackedWidget->setCurrentWidget(ui.pageplayer);
ui.showlistButton->setEnabled(false);
ui.playlistWidget->setSelectionMode (QAbstractItemView::SingleSelection);
ui.playlistWidget->setViewMode(QListView::ListMode);
ui.playlistWidget->setToolTip("Double click to play the media");
connect(ui.playlistWidget,SIGNAL(itemDoubleClicked(QListWidgetItem*)),this,SLOT(doubleclick(QListWidgetItem*)));

resizePlayer(440,0);

playtimer = new QTimer(this);
connect(playtimer, SIGNAL(timeout()), this, SLOT(updatePosition()));
connect(this, SIGNAL(playdone()), playtimer, SLOT(stop()));

username = QString(getenv("USER"));
initialize();
show();
}

qxmp::~qxmp(){
if (mp && mp->state()==QProcess::Running) 
	{
	mp->disconnect();
	mp->kill(); 
	delete mp ;
	}
if (proc && proc->state()==QProcess::Running) 
	{
	proc->disconnect();
	proc->kill(); 
	delete proc ;
	}
if (dload && dload->state()==QProcess::Running) 
	{
	dload->disconnect();
	dload->kill(); 
	delete dload ;
	}
}

void qxmp::closeEvent(QCloseEvent *e)
{
saveSettings();
if (mp && mp->state()==QProcess::Running) 
    {
    QString cmd="quit\n"; 
    mp->write(cmd.toAscii());
    }
e->accept();
}
//******************************************************************
void qxmp::playNextMedia()
{
if ( mediaList.count()>0 && mediaListPosition< mediaList.count()-1 && mediaListPosition>=-1)
	{
	mediaListPosition+=1;
	getMediaInfo();
	}
}
//****************************************************************
void qxmp::load(QString fn)
{
QString line,name;
QStringList mediaItem;
QDomDocument doc;
QDomElement root, element;
QDomNode node, subnode;
QDomNodeList nodelist;
QRegExp rxextinf("#EXTINF:\\d+,(.*)");
QRegExp rxfile("^\\s*File\\d+\\s*=(.*)");
QRegExp rxtitle("^\\s*Title\\d+\\s*=(.*)");
QRegExp rxml(".*(http://|mms://|rtsp://|pnm://)(.*)(\\.mpg|\\.mpeg|\\.wmv|\\.avi|\\.asf|\\.mov|\\.rm|\\.divx|\\.mkv|\\.ogm|\\.mp3|\\.wav|\\.ogg|\\.mka|\\.wma|\\.mp2|\\.ra |\\.m3u|\\.pls|\\.ram|\\.smi|\\.asx|\\.wax|\\.wvx)");
if (mp && mp->state()==QProcess::Running) return;
if (fn.isEmpty()) return;
if (fn.contains(QRegExp("\\.m3u|\\.pls|\\.ram|\\.smi|\\.asx|\\.wax|\\.wvx")) && fn.contains("http://")) 
	{
	if (!fn.contains("qxmp_temp")) previousMedia=fn;
	downloadPlaylist(fn);
	}
else if (fn.right(4).toLower()==".m3u")
	{
	QFile file(fn);
	if (!file.open(QIODevice::ReadOnly))
		{
		initialize();
		showTitle("Media Error");
		return;
		}
    	QTextStream stream(&file);
	mediaType=ISPLAYLIST;
	mediaList.clear();
	ui.playlistWidget->clear();
	mediaListPosition=-1;
	while (!stream.atEnd()) 
		{
		line = stream.readLine();
		name="";
		while (line.startsWith("#") || line.isEmpty())
			{
			if (rxextinf.indexIn(line)>-1) name =rxextinf.cap(1);
			line = stream.readLine();
			}
		line=line.remove("file://");
		mediaList.append(line);
		QListWidgetItem *newItem = new QListWidgetItem(ui.playlistWidget);
		if (name.isEmpty())
			{
			if (!QFileInfo(line).fileName().isEmpty()) newItem->setText(QFileInfo(line).fileName());
			else newItem->setText(line);
			}
		else newItem->setText(name);
		if (line.lastIndexOf(QRegExp("\\.mp3|\\.wav|\\.ogg|\\.mka|\\.wma|\\.mp2|\\.ra|\\.asf"))>-1) newItem->setIcon(QIcon(":/images/smallmusic.png"));
		else newItem->setIcon(QIcon(":/images/smallvideo.png"));
		newItem->setFlags(Qt::ItemIsEnabled);
		}
	ui.showlistButton->setEnabled(true);
	file.close();
	previousPosition=-1;
	if (!fn.contains("qxmp_temp")) previousMedia=fn;
	playNextMedia();
	}
else if (fn.right(4).toLower()==".pls")
	{
	QFile file(fn);
	if (!file.open(QIODevice::ReadOnly))
		{
		initialize();
		showTitle("Media Error");
		return;
		}
    	QTextStream stream(&file);
	mediaType=ISPLAYLIST;
	mediaList.clear();
	mediaItem.clear();
	ui.playlistWidget->clear();
	mediaListPosition=-1;
	while (!stream.atEnd()) 
		{
		line = stream.readLine();
		if (rxfile.indexIn(line)>-1) mediaList.append(rxfile.cap(1).remove("file://"));
		if (rxtitle.indexIn(line)>-1) mediaItem.append(line);
		}
	ui.showlistButton->setEnabled(true);
	for (int i = 0; i < mediaList.count(); i++)
		{
		QListWidgetItem *newItem = new QListWidgetItem(ui.playlistWidget);
		QRegExp rxti("^\\s*Title"+QString::number(i+1)+"\\s*=(.*)");
		if (mediaItem.indexOf(rxti)>-1) 
			{
			newItem->setText(rxti.cap(1));
			}
		else
			{
			name=QFileInfo(mediaList.at(i)).fileName();
			if (!name.isEmpty()) newItem->setText(name);
			else newItem->setText(mediaList.at(i));
			}
		if (mediaList.at(i).lastIndexOf(QRegExp("\\.mp3|\\.wav|\\.ogg|\\.mka|\\.wma|\\.mp2|\\.ra|\\.asf"))>-1) newItem->setIcon(QIcon(":/images/smallmusic.png"));
		else newItem->setIcon(QIcon(":/images/smallvideo.png"));
		newItem->setFlags(Qt::ItemIsEnabled);
		}
	file.close();
	previousPosition=-1;
	if (!fn.contains("qxmp_temp")) previousMedia=fn;
	playNextMedia();
	}
else if (fn.right(4).toLower()==".ram")
	{
	QFile file(fn);
	if (!file.open(QIODevice::ReadOnly))
		{
		initialize();
		showTitle("Media Error");
		return;
		}
    	QTextStream stream(&file);
	mediaType=ISPLAYLIST;
	mediaList.clear();
	ui.playlistWidget->clear();
	mediaListPosition=-1;
	while (!stream.atEnd()) 
		{
		line = stream.readLine();
		while (line.startsWith("#") || line.isEmpty()) line = stream.readLine();
		mediaList.append(line.remove("file://"));
		}
	ui.showlistButton->setEnabled(true);
	for (int i = 0; i < mediaList.count(); i++)
		{
		QListWidgetItem *newItem = new QListWidgetItem(ui.playlistWidget);
		name=QFileInfo(mediaList.at(i)).fileName();
		if (!name.isEmpty()) newItem->setText(name);
		else newItem->setText(mediaList.at(i));
		if (mediaList.at(i).lastIndexOf(QRegExp("\\.mp3|\\.wav|\\.ogg|\\.mka|\\.wma|\\.mp2|\\.ra|\\.asf"))>-1) newItem->setIcon(QIcon(":/images/smallmusic.png"));
		else newItem->setIcon(QIcon(":/images/smallvideo.png"));
		newItem->setFlags(Qt::ItemIsEnabled);
		}
	file.close();
	previousPosition=-1;
	if (!fn.contains("qxmp_temp")) previousMedia=fn;
	playNextMedia();
	}
else if (fn.right(4).toLower()==".asx" || fn.right(4).toLower()==".wax" || fn.right(4).toLower()==".wvx")
	{
	QFile file(fn);
	if (!file.open(QIODevice::ReadOnly))
		{
		initialize();
		showTitle("Media Error");
		return;
		}
	if (!doc.setContent(&file))
		{
		initialize();
		showTitle("Media Error");
		return;
		}
	mediaType=ISPLAYLIST;
	mediaList.clear();
	ui.playlistWidget->clear();
	mediaListPosition=-1;
	root = doc.documentElement();
	node = root.firstChild();
	while (!node.isNull())
		{
		line ="";
		name ="";
		if (node.nodeName().toLower() == "entry")
			{
			subnode = node.firstChild();
			while (!subnode.isNull())
				{
				if ((subnode.nodeName().toLower() == "ref") && (subnode.isElement()) && (line.isEmpty()))
					{
					element = subnode.toElement();
					if (element.hasAttribute("href")) line = element.attribute("href");
					if (element.hasAttribute("HREF")) line = element.attribute("HREF");
					if (element.hasAttribute("Href")) line = element.attribute("Href");
					if (element.hasAttribute("HRef")) line = element.attribute("HRef");
					}
				if ((subnode.nodeName().toLower() == "title") && (subnode.isElement())) name = subnode.toElement().text();
				subnode = subnode.nextSibling();
				}
			if (!line.isEmpty())
				{
				line=line.remove("file://");
				mediaList.append(line);
				QListWidgetItem *newItem = new QListWidgetItem(ui.playlistWidget);
				if (name.isEmpty())
					{
					if (!QFileInfo(line).fileName().isEmpty()) newItem->setText(QFileInfo(line).fileName());
					else newItem->setText(line);
					}
				else newItem->setText(name);
				if (line.lastIndexOf(QRegExp("\\.mp3|\\.wav|\\.ogg|\\.mka|\\.wma|\\.mp2|\\.ra|\\.asf"))>-1) newItem->setIcon(QIcon(":/images/smallmusic.png"));
				else newItem->setIcon(QIcon(":/images/smallvideo.png"));
				newItem->setFlags(Qt::ItemIsEnabled);
				}
			}
		node = node.nextSibling();
		}
	file.close();
	ui.showlistButton->setEnabled(true);
	previousPosition=-1;
	if (!fn.contains("qxmp_temp")) previousMedia=fn;
	playNextMedia();
	}
else if (fn.right(4).toLower()==".smi")
	{
	QFile file(fn);
	if (!file.open(QIODevice::ReadOnly))
		{
		initialize();
		showTitle("Media Error");
		return;
		}
	if (!doc.setContent(&file))
		{
		initialize();
		showTitle("Media Error");
		return;
		}
	mediaType=ISPLAYLIST;
	mediaList.clear();
	ui.playlistWidget->clear();
	mediaListPosition=-1;
	nodelist = doc.elementsByTagName("video");
	for (int i = 0; i < nodelist.count(); i++)
		{
		node = nodelist.item(i);
		line ="";
		if ((node.nodeName().toLower() == "video") && (node.isElement()))
			{
			element = node.toElement();
			if (element.hasAttribute("src")) line = element.attribute("src");
			if (element.hasAttribute("Src")) line = element.attribute("Src");
			if (element.hasAttribute("SRC")) line = element.attribute("SRC");
			}
		if (!line.isEmpty()) mediaList.append(line.remove("file://"));
		}
	nodelist = doc.elementsByTagName("audio");
	for (int i = 0; i < nodelist.count(); i++)
		{
		node = nodelist.item(i);
		line ="";
		if ((node.nodeName().toLower() == "audio") && (node.isElement()))
			{
			element = node.toElement();
			if (element.hasAttribute("src")) line = element.attribute("src");
			if (element.hasAttribute("Src")) line = element.attribute("Src");
			if (element.hasAttribute("SRC")) line = element.attribute("SRC");
			}
		if (!line.isEmpty()) mediaList.append(line.remove("file://"));
		}
	for (int i = 0; i < mediaList.count(); i++)
		{
		QListWidgetItem *newItem = new QListWidgetItem(ui.playlistWidget);
		name=QFileInfo(mediaList.at(i)).fileName();
		if (!name.isEmpty()) newItem->setText(name);
		else newItem->setText(mediaList.at(i));
		if (mediaList.at(i).lastIndexOf(QRegExp("\\.mp3|\\.wav|\\.ogg|\\.mka|\\.wma|\\.mp2|\\.ra|\\.asf"))>-1) newItem->setIcon(QIcon(":/images/smallmusic.png"));
		else newItem->setIcon(QIcon(":/images/smallvideo.png"));
		newItem->setFlags(Qt::ItemIsEnabled);
		}
	file.close();
 	ui.showlistButton->setEnabled(true);
	previousPosition=-1;
	if (!fn.contains("qxmp_temp")) previousMedia=fn;
	playNextMedia();
	}
else if (fn.right(4).toLower()==".xml" || fn.right(4).toLower()==".htm" || fn.right(5).toLower()==".html")
	{
	QFile file(fn);
	if (!file.open(QIODevice::ReadOnly))
		{
		initialize();
		showTitle("Media Error");
		return;
		}
    	QTextStream stream(&file);
	mediaType=ISPLAYLIST;
	mediaList.clear();
	ui.playlistWidget->clear();
	mediaListPosition=-1;
	while (!stream.atEnd()) 
		{
		line = stream.readLine();
		if (rxml.indexIn(line)>-1) mediaList.append(rxml.cap(1)+rxml.cap(2)+rxml.cap(3));
		}
	ui.showlistButton->setEnabled(true);
	for (int i = 0; i < mediaList.count(); i++)
		{
		QListWidgetItem *newItem = new QListWidgetItem(ui.playlistWidget);
		name=QFileInfo(mediaList.at(i)).fileName();
		if (!name.isEmpty()) newItem->setText(name);
		else newItem->setText(mediaList.at(i));
		if (mediaList.at(i).lastIndexOf(QRegExp("\\.mp3|\\.wav|\\.ogg|\\.mka|\\.wma|\\.mp2|\\.ra|\\.asf"))>-1) newItem->setIcon(QIcon(":/images/smallmusic.png"));
		else newItem->setIcon(QIcon(":/images/smallvideo.png"));
		newItem->setFlags(Qt::ItemIsEnabled);
		}
	file.close();
	previousPosition=-1;
	if (!fn.contains("qxmp_temp")) previousMedia=fn;
	playNextMedia();
	}
else
	{
	mediaType=ISFILE;
	mediaList.clear();
	ui.playlistWidget->clear();
	ui.showlistButton->setEnabled(false);
	mediaListPosition=-1;
	fn.remove("file://");
	mediaList.append(fn);
	if (!fn.contains("qxmp_temp")) previousMedia=fn;
	playNextMedia();
	}
}

void qxmp::loadFile()
{
if (mp && mp->state()==QProcess::Running) return;
QString fn=QFileDialog::getOpenFileName(this,"Select a File",lastDir,"Media Files (*.mpg *.mpeg *.wmv *.avi *.asf *.mov *.rm *.divx *.mkv *.ogm *.mp3 *.wav *.ogg *.mka *.wma *.mp2 *.ra *.m3u *.pls *.ram *.smi *.asx *.wax *.wvx);;All files (*.*)");
QFileInfo fi(fn);
if (fi.exists() && fi.isReadable()) 
	{
	lastDir=fi.absolutePath();
	load(fn);
	}
}

void qxmp::loadURL()
{
if (mp && mp->state()==QProcess::Running) return;
QString fn;
urlDlg = new UrlDialog(this);
if (urlDlg->exec())
	{
	fn=urlDlg->ui.lineEditURL->text();
	}
delete urlDlg;
load(fn);
}

void qxmp::loadDVD()
{
if (mp && mp->state()==QProcess::Running) return;
mediaType=ISDVD;
ui.showlistButton->setEnabled(false);
mediaList.clear();
ui.playlistWidget->clear();
mediaListPosition=-1;
mediaList.append("DVD");
previousMedia="";
playNextMedia();
}

void qxmp::loadCDDA()
{
if (mp && mp->state()==QProcess::Running) return;
mediaType=ISCDDA;
ui.showlistButton->setEnabled(false);
mediaList.clear();
ui.playlistWidget->clear();
mediaListPosition=-1;
mediaList.append("CDDA");
previousMedia="";
playNextMedia();
}

//*****************************************************************
void qxmp::playerPause()
{
if (mp && mp->state()==QProcess::Running) 
	{
	moviePause=!moviePause;
	QString cmd="pause\n"; 
	mp->write(cmd.toAscii());
	if (moviePause)
		{
		if (!isStreamCdda) 
			{
			ui.fwdButton->setEnabled(false);
			fwdAction->setEnabled(false);
			}
		if (!isStreamCdda) 
			{
			ui.rwdButton->setEnabled(false);
			rwdAction->setEnabled(false);
			}
		ui.soundButton->setEnabled(false);
		muteAction->setEnabled(false);
		ui.soundSlider->setDisabled(true);
		}
	else
		{
		if (!isStreamCdda) 
			{
			ui.fwdButton->setEnabled(true);
			fwdAction->setEnabled(true);
			}
		if (!isStreamCdda) 
			{
			ui.rwdButton->setEnabled(true);
			rwdAction->setEnabled(true);
			}
		ui.soundButton->setEnabled(true);
		muteAction->setEnabled(true);
		ui.soundSlider->setDisabled(false);
		}
	}
}

void qxmp::playerStop()
{
if (mp && mp->state()==QProcess::Running) 
	{
	mediaListPosition=mediaList.count()-1;
	QString cmd="quit\n"; 
	mp->write(cmd.toAscii());
	}
if (proc && proc->state()==QProcess::Running) 
	{
	proc->disconnect();
	proc->kill(); 
	initialize();
	showTitle("Welcome...");
	}
if (dload && dload->state()==QProcess::Running) 
	{
	dload->disconnect();
	dload->kill(); 
	initialize();
	showTitle("Welcome...");
	}
}

void qxmp::playerMute()
{
if (mp && mp->state()==QProcess::Running) 
	{
	movieSound=!movieSound;
	if (movieSound) 
		{
		ui.soundButton->setImages("sound");
		ui.soundSlider->setDisabled(false);
		}
	else 
		{
		ui.soundButton->setImages("nosound");
		ui.soundSlider->setDisabled(true);
		}
	QString cmd="mute\n"; 
	mp->write(cmd.toAscii()); 
	}
}

void qxmp::playerReplay()
{
if (mp && mp->state()==QProcess::Running) return;
if (mediaType==ISDVD) 
	{
	loadDVD();
	return;
	}
else if (mediaType==ISCDDA) 
	{
	loadCDDA();
	return;
	}
else if ( mediaList.count()>0)
	{
	mediaListPosition=-1;
	playNextMedia();
	}
}

void qxmp::playerReward()
{
bool ok;
int m=moviePosition.toInt(&ok);
if (mp && mp->state()==QProcess::Running && !moviePause) 
	{
	QString cmd="seek "+QString::number(qMax(m-10,0))+" 1\n"; 
	mp->write(cmd.toAscii()); 
	}
}

void qxmp::playerForward()
{
bool ok;
int m=moviePosition.toInt(&ok);
if (mp && mp->state()==QProcess::Running && !moviePause) 
	{
	QString cmd="seek "+QString::number(qMin(m+10,100))+" 1\n"; 
	mp->write(cmd.toAscii()); 
	}
}

void qxmp::playerNext()
{
if (mp && mp->state()==QProcess::Running) 
	{
	if (mediaType==ISDVD || mediaType==ISCDDA) nextTrack=true;
	QString cmd="quit\n"; 
	mp->write(cmd.toAscii());
	}
}

void qxmp::playerPrevious()
{
if (mp && mp->state()==QProcess::Running) 
	{
	mediaListPosition-=2;
	if (mediaType==ISDVD || mediaType==ISCDDA) nextTrack=true;
	QString cmd="quit\n"; 
	mp->write(cmd.toAscii());
	}
}

//*******************************************************
int qxmp::getVolume()
{
int vol = 0;
int res = -1;
res = ioctl (_mixerfd, MIXER_READ(SOUND_MIXER_PCM), &vol);
if (res==-1) return 0;
return vol & 0x7f;
}

void qxmp::setVolume (int v)
{
int vol = 0;
int res = -1;
if (v <= 0) v = 1;
vol = ((v << 8) & 0x7f00) | (v & 0x007f);
res = ioctl (_mixerfd, MIXER_WRITE(SOUND_MIXER_PCM), &vol);
}

void qxmp::updatePosition()
{
bool ok;
QString cmd;
if (mp && mp->state()==QProcess::Running && !moviePause) 
	{
	cmd="get_time_pos\nget_percent_pos\n"; 
	mp->write(cmd.toAscii());
	}
QString timetext ("--:--");
int hour = qMin(movieTime.toInt(&ok) / 3600,99);
int min = (movieTime.toInt(&ok) % 3600) / 60;
int sec = movieTime.toInt(&ok) % 60;
int pos= moviePosition.toInt(&ok);
if (hour > 0) timetext.sprintf ("%d:%02d:%02d", hour, min, sec);
else timetext.sprintf ("%02d:%02d", min, sec);
if (streamCache!="0") ui.labelStatus->setText("Buffering... : "+streamCache);
ui.labelStatus->setText("Playing...");
ui.labelTime->setText(timetext);
if (mediaType!=ISCDDA) ui.progressBar->setValue(pos);
}
//********************************************************************************************
void qxmp::startPlayer()
{
ui.progressBar->setValue(0);
bool ok;
int nbtracks;
if (errorProcess) 
	{
	if ( mediaList.count()==0 || mediaListPosition>=mediaList.count()-1 ) initialize();
	else
		{
		initialButtonState();
		ui.labelStatus->setText("Ready...");
		ui.labelTime->setText("");
		ui.progressBar->setValue(0);
		emit playdone();
		}
	showTitle("Media Error");
	return;
	}
if (mediaList.at(mediaListPosition)=="DVD")
	{
	mediaList.clear();
	ui.playlistWidget->clear();
	mediaListPosition=0;
	nbtracks=dvdChapters.toInt(&ok);
	if (nbtracks<1)
		{
		initialize();
		showTitle("Media Error");
		return;
		}
	ui.showlistButton->setEnabled(true);
	for (int i = 0; i < nbtracks; i++)
		{
		mediaList.append(dvdOptions+" -chapter "+QString::number(i+1)+"-"+QString::number(nbtracks));
		QListWidgetItem *newItem = new QListWidgetItem(ui.playlistWidget);
		newItem->setText("Start playing at chapter "+QString::number(i+1));
		newItem->setIcon(QIcon(":/images/smallvideo.png"));
		newItem->setFlags(Qt::ItemIsEnabled);
		}
	previousPosition=0;
	}

if (mediaList.at(mediaListPosition)=="CDDA")
	{
	mediaList.clear();
	ui.playlistWidget->clear();
	mediaListPosition=0;
	nbtracks=cddaTracks.toInt(&ok);
	if (nbtracks<1)
		{
		initialize();
		showTitle("Media Error");
		return;
		}
	ui.showlistButton->setEnabled(true);
	for (int i = 0; i < nbtracks; i++)
		{
		mediaList.append(cddaOptions+" cdda://"+QString::number(i+1)+"-"+QString::number(nbtracks));
		QListWidgetItem *newItem = new QListWidgetItem(ui.playlistWidget);
		newItem->setText("Start playing at track "+QString::number(i+1));
		newItem->setIcon(QIcon(":/images/smallmusic.png"));
		newItem->setFlags(Qt::ItemIsEnabled);
		}
	previousPosition=0;
	}

QString command, title, name;
QString playfile=mediaList.at(mediaListPosition);

ui.labelStatus->setText("Playing...");
playingButtonState();
ui.soundSlider->setDisabled(false);
isStreamCdda=false;

QString timetext ("--:--");
if (movieLength!="0")
	{
	int hour = movieLength.toInt(&ok) / 3600;
	int min = (movieLength.toInt(&ok) % 3600) / 60;
	int sec = movieLength.toInt(&ok) % 60;
	if (hour > 0) timetext.sprintf ("%d:%02d:%02d", hour, min, sec);
	else timetext.sprintf ("%02d:%02d", min, sec);
	timetext=" ["+timetext+"]";
	}
else timetext=" ";

switch (mediaType)
{
	case ISFILE:
	{
	if (mediaTitle.isEmpty()) 
		{
		name=QFileInfo(playfile).fileName();
		if (!name.isEmpty()) title=name;
		else title=playfile; 
		}
	else title=mediaTitle;
	title+=timetext;
	if (!mediaArtist.isEmpty()) title+=" - "+mediaArtist;
	if (!mediaAlbum.isEmpty()) title+=" - "+mediaAlbum;
	if (!mediaGenre.isEmpty()) title+=" - "+mediaGenre;
	if ( playfile.contains(QRegExp("http://|mms://|rtsp://|pnm://")))
		{
		ui.fwdButton->setEnabled(false);
		fwdAction->setEnabled(false);
		ui.rwdButton->setEnabled(false);
		rwdAction->setEnabled(false);
		isStreamCdda=true;
		}
	playfile="\""+playfile+"\"";
	}break;
	case ISPLAYLIST:
	{
	if (mediaTitle.isEmpty()) 
		{
		name=ui.playlistWidget->item(mediaListPosition)->text();
		if (!name.isEmpty()) title=name;
		else
			{
			name=QFileInfo(playfile).fileName();
			if (!name.isEmpty()) title=name;
			else title=playfile; 
			}
		}
	else title=mediaTitle;
	title+=timetext+" #"+QString::number(mediaListPosition+1)+"/"+QString::number(mediaList.count());
	if (!mediaArtist.isEmpty()) title+=" - "+mediaArtist;
	if (!mediaAlbum.isEmpty()) title+=" - "+mediaAlbum;
	if (!mediaGenre.isEmpty()) title+=" - "+mediaGenre;
	if ( playfile.contains(QRegExp("http://|mms://|rtsp://|pnm://")))
		{
		ui.fwdButton->setEnabled(false);
		fwdAction->setEnabled(false);
		ui.rwdButton->setEnabled(false);
		rwdAction->setEnabled(false);
		isStreamCdda=true;
		}
	playfile="\""+playfile+"\"";
	if (previousPosition>-1)
		{
		if (mediaList.at(previousPosition).lastIndexOf(QRegExp("\\.mp3|\\.wav|\\.ogg|\\.mka|\\.wma|\\.mp2|\\.ra|\\.asf"))>-1) ui.playlistWidget->item(previousPosition)->setIcon(QIcon(":/images/smallmusic.png"));
		else ui.playlistWidget->item(previousPosition)->setIcon(QIcon(":/images/smallvideo.png"));
		}
	ui.playlistWidget->setCurrentRow(mediaListPosition);
	ui.playlistWidget->item(mediaListPosition)->setIcon(QIcon(":/images/smallplay.png"));
	previousPosition=mediaListPosition;
	}break;
	case ISDVD:
	{
	title="DVD"+timetext+" - Titles : "+dvdTitles+" - Chapters : "+dvdChapters+" - Format : "+movieAspect;
	if (previousPosition>-1) ui.playlistWidget->item(previousPosition)->setIcon(QIcon(":/images/smallvideo.png"));
	ui.playlistWidget->setCurrentRow(mediaListPosition);
	ui.playlistWidget->item(mediaListPosition)->setIcon(QIcon(":/images/smallplay.png"));
	previousPosition=mediaListPosition;
	nextTrack=false;
	}break;
	case ISCDDA:
	{
	title="Audio CD"+timetext+" - Tracks : "+cddaTracks;
	ui.fwdButton->setEnabled(false);
	fwdAction->setEnabled(false);
	ui.rwdButton->setEnabled(false);
	rwdAction->setEnabled(false);
	isStreamCdda=true;
	if (previousPosition>-1) ui.playlistWidget->item(previousPosition)->setIcon(QIcon(":/images/smallmusic.png"));
	ui.playlistWidget->setCurrentRow(mediaListPosition);
	ui.playlistWidget->item(mediaListPosition)->setIcon(QIcon(":/images/smallplay.png"));
	previousPosition=mediaListPosition;
	nextTrack=false;
	}break;
}
showTitle(title);

int w=movieWidth.toInt( &ok );
int h=movieHeight.toInt( &ok );
moviePause=false;
moviePosition="0";
movieTime="0";
streamCache="0";
QString otherOptions="";
if (movieAspect!="0") otherOptions="-aspect "+movieAspect+" ";
QRect screen=QApplication::desktop()->screenGeometry();
otherOptions+="-monitoraspect "+QString::number(screen.width())+":"+QString::number(screen.height())+" ";
if ( playfile.contains("mms://")) otherOptions+="-cache 1024 ";
switch (viewmode)
{
	case 1:
	{
	if (w>0 && h>0) resizePlayer(w,h);
	else resizePlayer(440,0);
	QX11EmbedContainer *container=new QX11EmbedContainer(ui.frame);
	container->resize(w,h);
	mp=new QProcess(container);
	connect(mp, SIGNAL(readyReadStandardOutput()),this, SLOT(readPlayOutput()));
	connect(mp, SIGNAL(finished(int)),this, SLOT(endPlayProcess(int)));
	connect(mp, SIGNAL(started()),this, SLOT(startTimer()));
	command=otherOptions+mplayerOptions+" -wid "+QString::number(container->winId())+" -slave "+playfile;
	mp->start("mplayer "+command);
	}break;
	case 2:
	{
	if (w>0 && h>0)
		{
		QRect screen = QApplication::desktop()->availableGeometry();
		float scalex=(float) (screen.width()-24-desktopmargin)/w;
		float scaley=(float) (screen.height()-132-desktopmargin)/h;
		float scale=qMin(scalex,scaley);
		w=(int) (w*scale);
		h=(int) (h*scale);
		resizePlayer(w,h);
		}
	else resizePlayer(440,0);
	QX11EmbedContainer *container=new QX11EmbedContainer(ui.frame);
	container->resize(w,h);
	mp=new QProcess(container);
	connect(mp, SIGNAL(readyReadStandardOutput()),this, SLOT(readPlayOutput()));
	connect(mp, SIGNAL(finished(int)),this, SLOT(endPlayProcess(int)));
	connect(mp, SIGNAL(started()),this, SLOT(startTimer()));
	command=otherOptions+mplayerOptions+" -wid "+QString::number(container->winId())+" -zoom -vf scale="+QString::number(w)+":"+QString::number(h)+" -slave "+playfile;
	mp->start("mplayer "+command);
	}break;
	case 3:
	{
	resizePlayer(440,0);
	mp=new QProcess(this);
	connect(mp, SIGNAL(readyReadStandardOutput()),this, SLOT(readPlayOutput()));
	connect(mp, SIGNAL(finished(int)),this, SLOT(endPlayProcess(int)));
	connect(mp, SIGNAL(started()),this, SLOT(startTimer()));
	command=otherOptions+mplayerOptions+" -slave -fs "+playfile;
	mp->start("mplayer "+command);
	}break;
}
qDebug() << command.toLatin1().data();
}

//********************************************************************************************
void qxmp::getMediaInfo()
{
ui.labelStatus->setText("Analyzing...");
analyseButtonState();
movieAspect="0";
movieLength="0";
movieWidth="0";
movieHeight="0";
dvdChapters="0";
dvdTitles="0";
cddaTracks="0";
mediaTitle="";
mediaArtist="";
mediaAlbum="";
mediaYear="";
mediaGenre="";
proc = new QProcess( this );
connect(proc, SIGNAL(readyReadStandardOutput()),this, SLOT(readInfoOutput()));
connect(proc, SIGNAL(readyReadStandardError()),this, SLOT(readInfoError()));
connect(proc, SIGNAL(finished(int)),this, SLOT(endInfoProcess(int)));
errorProcess=false;
switch (mediaType)
{
	case ISFILE:
	{
	proc->start("mplayer -vo null -ao null -frames 0 -cache 128 -identify \""+mediaList.at(mediaListPosition)+"\"");
	}break;
	case ISPLAYLIST:
	{
	proc->start("mplayer -vo null -ao null -frames 0 -cache 128 -identify \""+mediaList.at(mediaListPosition)+"\"");
	}break;
	case ISDVD:
	{
	proc->start("mplayer -vo null -ao null -frames 0 -identify "+dvdOptions);
	}break;
	case ISCDDA:
	{
	proc->start("mplayer -vo null -ao null -frames 0 -identify "+cddaOptions+ " cdda://");
	}break;
}
}

//********************************************************************************************
void qxmp::endInfoProcess(int err)
{
if (err) errorProcess=true;
initialButtonState();
emit infodone();
}

void qxmp::readInfoError()
{
QByteArray result=proc->readAllStandardError();
QRegExp rxerror( "File not found:|Couldn't open DVD device:|Can't open disc|FATAL" );
if (rxerror.indexIn(QString(result))>-1)  errorProcess=true;
}

void qxmp::readInfoOutput()
{
QByteArray result=proc->readAllStandardOutput ();
QString line;
QTextStream stream(&result);
while (!stream.atEnd()) 
	{
	line = stream.readLine();
	// movie dimension
	QRegExp rxw( "ID_VIDEO_WIDTH=(\\d+)" );
	if (rxw.indexIn(line)>-1) movieWidth = rxw.cap( 1 );
	QRegExp rxh( "ID_VIDEO_HEIGHT=(\\d+)" );
	if (rxh.indexIn(line)>-1) movieHeight = rxh.cap( 1 );
	// best dimension if available
	QRegExp rxdim( "VO:.*=>\\s(\\d+)x(\\d+)" );
	if (rxdim.indexIn(line)>-1) 
		{
		movieWidth = rxdim.cap( 1 );
		movieHeight = rxdim.cap( 2 );
		}
	// movie length
	QRegExp rxlength( "ID_LENGTH=(\\d+)" );
	if (rxlength.indexIn(line)>-1) movieLength = rxlength.cap( 1 );
	// movie aspect
	QRegExp rxaspect( "Movie\\-Aspect\\sis\\s=*([0-9]+[.,]?[0-9]*:[0-9]+[.,]?[0-9]*)" );
	if (rxaspect.indexIn(line)>-1) movieAspect = rxaspect.cap( 1 );
	
	QRegExp rxdvdchap( "There are ([0-9]+) chapters" );
	if (rxdvdchap.indexIn(line)>-1) dvdChapters = rxdvdchap.cap( 1 );
	QRegExp rxdvdtit( "There are ([0-9]+) titles" );
	if (rxdvdtit.indexIn(line)>-1) dvdTitles = rxdvdtit.cap( 1 );
	
	//media Info
	QRegExp rxtitle( "Title:\\s(.*)" );
	if (rxtitle.indexIn(line)>-1) mediaTitle=rxtitle.cap(1);
	QRegExp rxartist( "Artist:\\s(.*)" );
	if (rxartist.indexIn(line)>-1) mediaArtist=rxartist.cap(1);
	QRegExp rxalbum( "Album:\\s(.*)" );
	if (rxalbum.indexIn(line)>-1) mediaAlbum=rxalbum.cap(1);
	QRegExp rxyear( "Year:\\s(.*)" );
	if (rxyear.indexIn(line)>-1) mediaYear=rxyear.cap(1);
	QRegExp rxgenre( "Genre:\\s(.*)" );
	if (rxgenre.indexIn(line)>-1) mediaGenre=rxgenre.cap(1);

	//cdda tracks
	QRegExp rxtracks( "ID_CDDA_TRACKS=(\\d+)" );
	if (rxtracks.indexIn(line)>-1) cddaTracks = rxtracks.cap( 1 );
	}
}
//********************************************************************************************
void qxmp::endPlayProcess(int err)
{
if ( mediaList.count()==0 || mediaListPosition>=mediaList.count()-1 ) initialButtonState();
moviePosition="100";
movieTime="0";
streamCache="0";
updatePosition();
ui.labelStatus->setText("Ready...");
ui.labelTime->setText("");
ui.progressBar->setValue(0);
QString result="Process exited with error(s)";
if (err) qDebug() << result.toLatin1().data();
if ((mediaType==ISDVD || mediaType==ISCDDA) && !nextTrack) mediaListPosition=mediaList.count()-1;
emit playdone();
}

void qxmp::readPlayOutput()
{
QByteArray result=mp->readAllStandardOutput ();
QRegExp rxw( "ANS_PERCENT_POSITION=(\\d+)" );
if (rxw.indexIn(QString(result))>-1) moviePosition = rxw.cap( 1 );
QRegExp rxt( "ANS_TIME_POSITION=(\\d+)" );
if (rxt.indexIn(QString(result))>-1) movieTime = rxt.cap( 1 );
QRegExp rxc( "Cache fill:\\s*([0-9]*.[0-9]*%)" );
if (rxc.indexIn(QString(result))>-1) streamCache = rxc.cap( 1 );
else streamCache="0";
}

void qxmp::startTimer()
{
playtimer->start(500);
}
//*********************************************************************
void qxmp::resizePlayer(int w, int h)
{
QRect screen=QApplication::desktop()->availableGeometry(); 
if (h==0)
	{
	ui.frame->hide();
	ui.stackedWidget->move(0,0);
	ui.stackedWidget->show();
	this->setMinimumSize(QSize(440,120));
	this->setMaximumSize(QSize(440,120));
	this->resize(440,120);
	}
else
	{
	ui.frame->move(qMax(450,w+4)/2-w/2,2);
	ui.frame->setMinimumSize(QSize(w+4,h+4));
	ui.frame->setMaximumSize(QSize(w+4,h+4));
	ui.frame->resize(QSize(w+4,h+4));
	ui.frame->show();
	ui.stackedWidget->move(qMax(450,w+8)/2-220,8+h);
	ui.stackedWidget->show();
	this->setMinimumSize(QSize(qMax(450,w+8),h+132));
	this->setMaximumSize(QSize(qMax(450,w+8),h+132));
	this->resize(qMax(450,w+8),h+132);
	this->move((screen.width()-this->width())/2,(screen.height()-this->height())/2);
	}
}
//********************************************************************
void qxmp::initialButtonState()
{
ui.pauseButton->setEnabled(false);
pauseAction->setEnabled(false);
ui.stopButton->setEnabled(false);
stopAction->setEnabled(false);
ui.soundButton->setEnabled(false);
muteAction->setEnabled(false);
ui.fwdButton->setEnabled(false);
fwdAction->setEnabled(false);
ui.rwdButton->setEnabled(false);
rwdAction->setEnabled(false);
if (mediaList.count()>0) ui.reloadButton->setEnabled(true);
else ui.reloadButton->setEnabled(false);
ui.playButton->setEnabled(true);
ui.dvdButton->setEnabled(true);
ui.cddaButton->setEnabled(true);
ui.urlButton->setEnabled(true);
ui.configButton->setEnabled(true);
ui.bookmarkButton->setEnabled(true);
ui.playlistButton->setEnabled(true);
ui.previousButton->setEnabled(false);
previousAction->setEnabled(false);
ui.nextButton->setEnabled(false);
nextAction->setEnabled(false);
}
void qxmp::playingButtonState()
{
ui.pauseButton->setEnabled(true);
pauseAction->setEnabled(true);
ui.stopButton->setEnabled(true);
stopAction->setEnabled(true);
ui.soundButton->setEnabled(true);
muteAction->setEnabled(true);
ui.fwdButton->setEnabled(true);
fwdAction->setEnabled(true);
ui.rwdButton->setEnabled(true);
rwdAction->setEnabled(true);
ui.reloadButton->setEnabled(false);
ui.playButton->setEnabled(false);
ui.dvdButton->setEnabled(false);
ui.cddaButton->setEnabled(false);
ui.urlButton->setEnabled(false);
ui.configButton->setEnabled(true);
ui.bookmarkButton->setEnabled(true);
ui.playlistButton->setEnabled(true);
if ( mediaList.count()>0 && mediaListPosition<=mediaList.count()-1 && mediaListPosition>=1)
	{
	ui.previousButton->setEnabled(true);
	previousAction->setEnabled(true);
	}
else 
	{
	ui.previousButton->setEnabled(false);
	previousAction->setEnabled(false);
	}
if ( mediaList.count()>0 && mediaListPosition< mediaList.count()-1 && mediaListPosition>=-1) 
	{
	ui.nextButton->setEnabled(true);
	nextAction->setEnabled(true);
	}
else 
	{
	ui.nextButton->setEnabled(false);
	nextAction->setEnabled(false);
	}
}
void qxmp::analyseButtonState()
{
ui.pauseButton->setEnabled(false);
pauseAction->setEnabled(false);
ui.stopButton->setEnabled(true);
stopAction->setEnabled(true);
ui.soundButton->setEnabled(false);
muteAction->setEnabled(false);
ui.fwdButton->setEnabled(false);
fwdAction->setEnabled(false);
ui.rwdButton->setEnabled(false);
rwdAction->setEnabled(false);
ui.reloadButton->setEnabled(false);
ui.playButton->setEnabled(false);
ui.dvdButton->setEnabled(false);
ui.cddaButton->setEnabled(false);
ui.urlButton->setEnabled(false);
ui.configButton->setEnabled(true);
ui.bookmarkButton->setEnabled(true);
ui.playlistButton->setEnabled(true);
ui.previousButton->setEnabled(false);
previousAction->setEnabled(false);
ui.nextButton->setEnabled(false);
nextAction->setEnabled(false);
}
//********************************************************************
void qxmp::readSettings()
{
QSettings config("xm1","qxmp");
config.beginGroup( "qxmp" );
mplayerOptions=config.value("MplayerOptions","-vo xv").toString();
dvdOptions=config.value("DvdOptions","-dvd-device /dev/dvd dvd://1 -alang en").toString();
cddaOptions=config.value("CddaOptions","-cdrom-device /dev/cdrom -cache 4096").toString();
mixerDevice=config.value("MixerDevice","/dev/mixer").toString();
viewmode=config.value("Viewmode",2).toInt();
bookmarkNameList=config.value("Boomarks Names").toStringList();
bookmarkLocationList=config.value("Boomarks Locations").toStringList();
desktopmargin=config.value("DesktopMargin",50).toInt();
lastDir=config.value("LastDir",QDir::homePath()).toString();
QFontDatabase fdb;
QStringList xf = fdb.families();
QString deft;
if (xf.contains("Arial",Qt::CaseInsensitive)) deft="Arial";
else if (xf.contains("Bitstream Vera Sans",Qt::CaseInsensitive)) deft="Bitstream Vera Sans";
else if (xf.contains("Luxi Sans",Qt::CaseInsensitive)) deft="Luxi Sans";
else deft=qApp->font().family();
displayFont=config.value("Font Family",deft).toString();
config.endGroup();
}

void qxmp::saveSettings()
{
QSettings config("xm1","qxmp");
config.beginGroup( "qxmp" );
config.setValue("MplayerOptions",mplayerOptions);
config.setValue("DvdOptions",dvdOptions);
config.setValue("CddaOptions",cddaOptions);
config.setValue("MixerDevice",mixerDevice);
config.setValue("Viewmode",viewmode);
config.setValue("Boomarks Names",bookmarkNameList); 
config.setValue("Boomarks Locations",bookmarkLocationList); 
config.setValue("DesktopMargin",desktopmargin); 
config.setValue("LastDir",lastDir);
config.setValue("Font Family",displayFont);
config.endGroup();
}
//**********************************************************
void qxmp::configurePlayer()
{
confDlg = new ConfigDialog(this);
confDlg->ui.lineEditMplayer->setText(mplayerOptions);
confDlg->ui.lineEditDVD->setText(dvdOptions);
confDlg->ui.lineEditCDDA->setText(cddaOptions);
confDlg->ui.lineEditMixer->setText(mixerDevice);
if (viewmode==1) confDlg->ui.radioButton1->setChecked(true);
if (viewmode==2) confDlg->ui.radioButton2->setChecked(true);
if (viewmode==3) confDlg->ui.radioButton3->setChecked(true);
confDlg->ui.spinBoxMargin->setValue(desktopmargin);
int ft=confDlg->ui.comboBoxFont->findText (displayFont , Qt::MatchExactly);
confDlg->ui.comboBoxFont->setCurrentIndex(ft);
confDlg->ui.groupBox->setFocus();
if (confDlg->exec())
	{
	mplayerOptions=confDlg->ui.lineEditMplayer->text();
	dvdOptions=confDlg->ui.lineEditDVD->text();
	cddaOptions=confDlg->ui.lineEditCDDA->text();
	mixerDevice=confDlg->ui.lineEditMixer->text();
	if (confDlg->ui.radioButton1->isChecked()) viewmode=1;
	if (confDlg->ui.radioButton2->isChecked()) viewmode=2;
	if (confDlg->ui.radioButton3->isChecked()) viewmode=3;
	desktopmargin=confDlg->ui.spinBoxMargin->value();
	displayFont=confDlg->ui.comboBoxFont->currentText();
	QFont font(displayFont,12);
	font.setBold(true);
	font.setItalic(false);
	font.setUnderline(false);
	font.setStrikeOut(false);
	font.setPixelSize(18);
	ui.labelStatus->setFont(font);
	ui.labelTime->setFont(font);
	titlewidget->setTitle(titlewidget->current,displayFont);
	}
delete confDlg;
}
//***********************************************************
void qxmp::manageBookmarks()
{
bmkDlg=new BookmarkDialog(this);
bmkDlg->nameList=bookmarkNameList;
bmkDlg->locationList=bookmarkLocationList;
bmkDlg->currentMovieLocation=previousMedia;
if (previousMedia.isEmpty()) bmkDlg->ui.currentButton->setEnabled(false);
else bmkDlg->ui.currentButton->setEnabled(true);
if (mp && mp->state()==QProcess::Running) bmkDlg->ui.playButton->setEnabled(false);
else bmkDlg->ui.playButton->setEnabled(true);
bmkDlg->Init();
if (bmkDlg->exec())
	{
	bookmarkNameList.clear();
	bookmarkLocationList.clear();
	bookmarkNameList=bmkDlg->nameList;
	bookmarkLocationList=bmkDlg->locationList;
	if (!bmkDlg->movieToPlay.isEmpty()) load(bmkDlg->movieToPlay);
	}
delete bmkDlg;
}

void qxmp::playlistEditor()
{
QString line;
plDlg=new PlaylistDialog(this);
if (mp && mp->state()==QProcess::Running) plDlg->ui.playButton->setEnabled(false);
else plDlg->ui.playButton->setEnabled(true);
if (plDlg->exec())
	{
	if (!plDlg->tempPlayFile.isEmpty())
		{
		QTextStream stream(&plDlg->tempPlayFile);
		mediaType=ISPLAYLIST;
		mediaList.clear();
		ui.playlistWidget->clear();
		mediaListPosition=-1;
		while (!stream.atEnd()) 
			{
			line = stream.readLine();
			while (line.startsWith("#") || line.isEmpty()) line = stream.readLine();
			mediaList.append(line);
			}
		ui.showlistButton->setEnabled(true);
		for (int i = 0; i < mediaList.count(); i++)
			{
			QListWidgetItem *newItem = new QListWidgetItem(ui.playlistWidget);
			newItem->setText(QFileInfo(mediaList.at(i)).fileName());
			if (mediaList.at(i).lastIndexOf(QRegExp("\\.mp3|\\.wav|\\.ogg|\\.mka|\\.wma|\\.mp2|\\.ra|\\.asf"))>-1) newItem->setIcon(QIcon(":/images/smallmusic.png"));
			else newItem->setIcon(QIcon(":/images/smallvideo.png"));
			newItem->setFlags(Qt::ItemIsEnabled);
			}
		previousMedia="";
		previousPosition=-1;
		playNextMedia();
		}
	}
delete plDlg;
}
//***********************************************************
void qxmp::showTitle(QString title)
{
titlewidget->setTitle(title,displayFont);
}
//*********************************************************
void qxmp::showPlaylist()
{
ui.stackedWidget->setCurrentWidget(ui.pageplaylist);
}

void qxmp::showPlayer()
{
ui.stackedWidget->setCurrentWidget(ui.pageplayer);
}

void qxmp::doubleclick(QListWidgetItem* item)
{
int pos=ui.playlistWidget->currentRow();
if (mp && mp->state()==QProcess::Running) 
	{
	mediaListPosition=pos-1;
	if (mediaType==ISDVD || mediaType==ISCDDA) nextTrack=true;
	QString cmd="quit\n"; 
	mp->write(cmd.toAscii());
	}
else if ( mediaList.count()>0)
	{
	mediaListPosition=pos-1;
	playNextMedia();
	}
}
//*********************************************************
void qxmp::downloadPlaylist(QString url)
{
ui.labelStatus->setText("Downloading... : ");
analyseButtonState();
if (url.contains(QRegExp( "http://(.*)\\.m3u" ))) downloadFile = "/tmp/qxmp_temp_"+username+".m3u";
if (url.contains(QRegExp( "http://(.*)\\.pls" ))) downloadFile = "/tmp/qxmp_temp_"+username+".pls";
if (url.contains(QRegExp( "http://(.*)\\.ram" ))) downloadFile = "/tmp/qxmp_temp_"+username+".ram";
if (url.contains(QRegExp( "http://(.*)\\.smi" ))) downloadFile = "/tmp/qxmp_temp_"+username+".smi";;
if (url.contains(QRegExp( "http://(.*)\\.asx" ))) downloadFile = "/tmp/qxmp_temp_"+username+".asx";
if (url.contains(QRegExp( "http://(.*)\\.wax" ))) downloadFile = "/tmp/qxmp_temp_"+username+".wax";
if (url.contains(QRegExp( "http://(.*)\\.wvx" ))) downloadFile = "/tmp/qxmp_temp_"+username+".wvx";
dload = new QProcess( this );
connect(dload, SIGNAL(finished(int)),this, SLOT(endDownloadProcess(int)));
dload->start("wget -O "+downloadFile+" "+url);
}

void qxmp::endDownloadProcess(int err)
{
initialButtonState();
ui.labelStatus->setText("Ready...");
ui.labelTime->setText("");
ui.progressBar->setValue(0);
if (err) 
	{
	qDebug() << QString("Download has failed").toLatin1().data();
	initialButtonState();
	showTitle("Download Error");
	}
else
	{
	load(downloadFile);
	}
}
//********************************************************
void qxmp::initialize()
{
mediaList.clear();
ui.playlistWidget->clear();
movieWidth="440";
movieHeight="268";
movieSound=true;
previousMedia="";
mediaListPosition=-1;
downloadFile ="";
streamCache="0";
initialButtonState();
ui.labelStatus->setText("Ready...");
ui.labelTime->setText("");
ui.progressBar->setValue(0);
}
//*********************************************************
void qxmp::aboutPlayer()
{
abDlg = new AboutDialog(this);
abDlg->exec();
delete abDlg;
}
