/***************************************************************************
 *   copyright       : (C) 2006 by Pascal Brachet                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef QXMP_H
#define QXMP_H

#include <QDialog>
#include <QProcess>
#include <QX11EmbedContainer>
#include <QPointer>
#include <QTimer>
#include <QListWidgetItem>
#include <QFont>
#include <QAction>

#include "ui_qxmp.h"

#include "configdialog.h"
#include "urldialog.h"
#include "bookmarkdialog.h"
#include "playlistdialog.h"
#include "titlearea.h"
#include "aboutdialog.h"

class qxmp : public QDialog  {
   Q_OBJECT
public:
	qxmp(QWidget *parent=0);
	~qxmp();
	Ui::qxmp ui;

void load(QString fn);

private slots:
void closeEvent(QCloseEvent *e);
void playNextMedia();
void loadFile();
void loadURL();
void loadDVD();
void loadCDDA();
void playerPause();
void playerStop();
void playerMute();
void playerReplay();
void playerForward();
void playerReward();
void playerPrevious();
void playerNext();
int getVolume();
void setVolume (int v);
void updatePosition();
void startPlayer();
void getMediaInfo();
void endInfoProcess(int err);
void readInfoError();
void readInfoOutput();
void endPlayProcess(int err);
void readPlayOutput();
void startTimer();
void resizePlayer(int w, int h);
void initialButtonState();
void playingButtonState();
void analyseButtonState();
void readSettings();
void saveSettings();
void configurePlayer();
void manageBookmarks();
void playlistEditor();
void aboutPlayer();
void showTitle(QString title);
void showPlaylist();
void showPlayer();
void doubleclick(QListWidgetItem* item);
void downloadPlaylist(QString url);
void endDownloadProcess(int err);
void initialize();

private:
enum {ISFILE=0,ISPLAYLIST=1,ISDVD=2,ISCDDA=3};
QPointer<QProcess> mp, proc, dload;
QX11EmbedContainer *container;
bool movieSound, errorProcess, moviePause;
QString movieWidth, movieHeight, movieLength, movieAspect, moviePosition, movieTime, dvdChapters, dvdTitles;
QString mediaTitle, mediaArtist, mediaAlbum, mediaYear, mediaGenre, cddaTracks;
QString mplayerOptions, dvdOptions, cddaOptions, mixerDevice;
QString streamCache;
QString downloadFile;
QString lastDir;
int viewmode;
QTimer *playtimer;
QStringList mediaList;
int mediaListPosition;
int mediaType;
int previousPosition;
int desktopmargin;
QString previousMedia;
int _mixerfd;
ConfigDialog *confDlg;
UrlDialog *urlDlg;
BookmarkDialog *bmkDlg;
PlaylistDialog *plDlg;
AboutDialog *abDlg;
QStringList bookmarkNameList, bookmarkLocationList;
TitleArea* titlewidget;
QString username;
QAction *pauseAction, *stopAction, *muteAction, *fwdAction, *rwdAction, *nextAction, *previousAction;
QString displayFont;
bool nextTrack;
bool isStreamCdda;
signals:
void playdone();
void infodone();
};


#endif
