/***************************************************************************
 *   copyright       : (C) 2006 by Pascal Brachet                          *
 *                                                                         *
 *   inspired by the amarok code - (C) by Mark Kretschmann                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <QtGui>
#include <QPainter>
#include <QFontMetrics>
#include <QPalette>
#include <QDebug>


#include "titlearea.h"

TitleArea::TitleArea(QWidget *parent, QString f)
    : QWidget(parent)
{
this->resize(QSize(parent->width(),15));
this->move(0,0);
QPalette palette;
palette.setColor(this->backgroundRole(), QColor(24,93,140));
palette.setColor(this->foregroundRole(), QColor(255,255,255));
this->setPalette(palette);
this->setAutoFillBackground(true);

titletimer=new QTimer(this);
connect(titletimer, SIGNAL( timeout() ), SLOT( doScroll() ) );
setTitle("Welcome...",f);
show();
titletimer->start(40);
}

void TitleArea::setTitle(QString t, QString f)
{
current=t;
QFont titleFont(f,12);
titleFont.setBold(true);
titleFont.setItalic(false);
titleFont.setUnderline(false);
titleFont.setStrikeOut(false);
titleFont.setPixelSize(12);
QFontMetrics fm(titleFont);
title=" "+t.simplified();
title+="     ";
scrollTitlePixmap=QPixmap(fm.width(title),fm.lineSpacing());
scrollTitlePixmap.fill(QColor(24,93,140));
QPainter p(&scrollTitlePixmap);
p.setPen( QColor(255,255,255) );
p.setFont(titleFont);
p.drawText(0,fm.ascent(),title);
p.end();
scrollBuffer=QPixmap(this->width(),fm.lineSpacing());
}

//inspired by the amarok code - (C) by Mark Kretschmann
void TitleArea::doScroll()
{
static int phase=0;
int w=scrollTitlePixmap.width();
int h=scrollTitlePixmap.height();
phase+=1;
if( phase >= w ) phase = 0;
int subs = 0;
int dx = 0;
int phase2 = phase;
QPainter p(&scrollBuffer);
while( dx < this->width() )
	{
	subs = -this->width();
	subs += dx + ( w - phase2 );
	if( subs < 0 ) subs = 0;
	p.drawPixmap(dx,0,scrollTitlePixmap,phase2, 0, w - phase2 - subs, h);
	dx     += w - phase2;
	phase2 += w - phase2;
	if( phase2 >= w ) phase2 = 0;
	}
p.end();
update();
}

void TitleArea::paintEvent(QPaintEvent *)
{
QPainter painter(this);
painter.drawPixmap(0,(this->height()-scrollBuffer.height())/2,scrollBuffer);
painter.end();
}



